﻿var path = require('path');
var util = require('util');
var async = require('async');
var verify = require("../util/verify");
var log = require("../util/log")(module);
var ArtifactInfo = require('../artifact/artifactinfo');
var Runner = require('./runner');
var WizziJob = require('./wizziJob');
var ArtifactPersister = require('./persister');
var ProductionContext = require('./context');

var ProductionManager = function (options) {
    this.options = options;
    this.artifactInfos = [];
    this.wizziJobConfigs = [];
    this.schemaModules = [];
    this.artifactModules = [];
    this.productionContext = new ProductionContext();
    log.setLevel(options.verbose || 0);
    this.___state = {};
    this.___state.models = {};
    this.___state.pman = this;
    this.___state.pcx = this.productionContext;
};

// bulk add of artifact infos
ProductionManager.prototype.setArtifactInfos = function () {
    throw new Error('Not implemented');
}

// add an artifact info
ProductionManager.prototype.addArtifactInfo = function (artifactInfo) {
    if (ArtifactInfo.isArtifactConfig(artifactInfo)) {
        this.artifactInfos.push(new ArtifactInfo(artifactInfo));
    } else if (ArtifactInfo.isArtifactInstance(artifactInfo)) {
        this.artifactInfos.push(artifactInfo);
    } else {
        throw new Error('ProductionManager.addArtifactInfo is not an artifact info: ' + util.inspect(artifactInfo));
    }
}

// add a wizzi Job
ProductionManager.prototype.addWizziJob = function (wizziJobConfig) {
    this.wizziJobConfigs.push(wizziJobConfig);
}

// register a schema module
ProductionManager.prototype.registerSchemaModule = function (schemaModule) {
    this.schemaModules.push(schemaModule);
}

// get a schema loader
ProductionManager.prototype.getSchemaLoader = function (schemaName) {
    // console.log('ProductionManager.getSchemaLoader.schemaName: ' + schemaName);
    var result = null;
    this.schemaModules.forEach(function (item) {
        if (result == null) {
            if (verify.isFunction(item[schemaName])) result = item[schemaName];
        }
    });
    if (!result) {
        log.error('getSchemaLoader. Cannot find schema loader: ' + schemaName);
        var availables = [];
        this.schemaModules.forEach(function (item) {
            for (key in item) {
                if (verify.isFunction(item[key])) availables.push(key);
            }
        });
        log.error('availables: ' + availables.join(', '));
    }
    return result;
}

// register an artifact module
ProductionManager.prototype.registerArtifactModule = function (artifactModule) {
    this.artifactModules.push(artifactModule);
}

// get an artifact generator
ProductionManager.prototype.getArtifactGenerator = function (artifactName) {
    // console.log('ProductionManager.getArtifactGenerator.artifactName: ' + artifactName);
    var result = null;
    this.artifactModules.forEach(function (item) {
        if (result == null) {
            if (verify.isFunction(item['getGenerator'])) {
                result = item['getGenerator'](artifactName);
            }
        }
    });
    if (!result) {
        log.error('getArtifactGenerator. Cannot find artifact generator: ' + artifactName);
    }
    return result;
}

ProductionManager.prototype.initialize = function (callback) {
    log.info('Initialize');
    this.___state = {};
    this.___state.models = {};
    this.___state.pman = this;
    this.___state.pcx = this.productionContext;
    var _this = this;
    this.wizziJobConfigs.forEach(function (item) {
        var job = new WizziJob(_this, item);
        job.getArtifactInfos(function (err, result) {
            if (err) { callback(err); return; }
            result.forEach(function (artInfo) {
                // console.log('ProductionManager.initialize.artInfo', util.inspect(artInfo, { depth: 2 }));
                _this.addArtifactInfo(artInfo);
            });
        })
        _this.artifactInfos.forEach(function (item) {
            item.initialize(_this);
            
        });
        callback(null, null);
    });
}

// run the production of all the artifacts
ProductionManager.prototype.run = function (callback) {
    log.info('Start run');
    var _this = this;
    _this.initialize(function (err, result) {
        if (err) { callback(err); return; }
        async.map(_this.artifactInfos, AsyncRunner.run, function (err, result) {
            if (err) { callback(err); return; }
            _this.artifactInfos = result;
            _this.terminate();
            log.info('End run');
            callback(null, result);
        });
    });
}

ProductionManager.prototype.terminate = function () {
    log.info('Terminate');
    //this.artifactInfos.forEach(function (item) {
    //    item.dump();
    //});
}

// persist the production results to the file system, 
// using artifact infos to build destination paths
ProductionManager.prototype.persistToFile = function (callback) {
    log.info('Start persistToFile');
    var _this = this;
    async.map(_this.artifactInfos, AsyncPersisterToFile.run, function (err, result) {
        if (err) { callback(err, null); return; }
        // console.log('ProductionManager.persistToFile.persisted artifacts', util.inspect(result, { depth: 1 }));
        log.info('End persistToFile');
        callback(null, result);
    });
}

ProductionManager.prototype.relPath = function (filepath) {
    return filepath.substr(this.options.basedir.length + 1);
}

var AsyncRunner = {
    run: function (artifactInfo, callback) {
        log.info('Started async run artifact: ' + artifactInfo.name);
        var runner = new Runner(artifactInfo);
        runner.run(function (err, result) {
            if (err) { callback(err, null); return; }
            log.info('Ended async run artifact: ' + artifactInfo.name);
            callback(null, result);
        });
    }
};

var AsyncPersisterToFile = {
    run: function (artifactInfo, callback) {
        log.info('Started async persist to file artifact: ' + artifactInfo.name);
        var persister = new ArtifactPersister(artifactInfo);
        persister.toFile(function (err, result) {
            if (err) { callback(err, null); return; }
            log.info('Ended async persist to file artifact: ' + artifactInfo.name);
            callback(null, result);
        });
    }
};

module.exports = ProductionManager;
